package ink.wgink.module.wechat.official.account.request;

import ink.wgink.module.wechat.enums.ErrorMsgEnum;
import ink.wgink.module.wechat.official.account.manager.OfficialAccountAccessTokenManager;
import ink.wgink.module.wechat.official.account.pojo.OfficialAccountAccessToken;
import ink.wgink.module.wechat.official.account.result.AccessTokenResult;
import ink.wgink.properties.wechat.official.account.OfficialAccountProperties;
import ink.wgink.util.request.AbstractRestTemplate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.client.HttpClientErrorException;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: AccessTokenRestTemplate
 * @Description:
 * @Author: wanggeng
 * @Date: 2021/4/28 5:31 上午
 * @Version: 1.0
 */
public class AccessTokenRestTemplate extends AbstractRestTemplate<AccessTokenResult, Map<String, Object>> {
    private static final Logger LOG = LoggerFactory.getLogger(AccessTokenRestTemplate.class);
    private OfficialAccountProperties officialAccountProperties;

    public AccessTokenRestTemplate(OfficialAccountProperties officialAccountProperties) {
        this.officialAccountProperties = officialAccountProperties;
    }

    @Override
    public Map<String, Object> postBody() {
        Map<String, Object> params = new HashMap<>(6);
        params.put("grant_type", officialAccountProperties.getGrantType());
        params.put("appid", officialAccountProperties.getAppId());
        params.put("secret", officialAccountProperties.getAppSecret());
        return params;
    }

    @Override
    public Class<AccessTokenResult> successResultType() {
        return AccessTokenResult.class;
    }

    @Override
    public void success200(AccessTokenResult result) {
        if (result.getErrcode() == ErrorMsgEnum.SUCCESS.getErrCode()) {
            OfficialAccountAccessToken officialAccountAccessToken = new OfficialAccountAccessToken();
            officialAccountAccessToken.setAccessToken(result.getAccess_token());
            officialAccountAccessToken.setLastUpdated(System.currentTimeMillis());
            OfficialAccountAccessTokenManager.getInstance().setAccessToken(officialAccountAccessToken);
            return;
        }
        if (result.getErrcode() == ErrorMsgEnum.SYSTEM_BUSY.getErrCode()) {
            LOG.error("系统繁忙，3s后重试");
            try {
                TimeUnit.SECONDS.sleep(3);
            } catch (InterruptedException e) {
                LOG.error(e.getMessage(), e);
            }
            this.post(officialAccountProperties.getAccessTokenUrl());
            return;
        }
        LOG.error(ErrorMsgEnum.getErrMsg(result.getErrcode()));
    }

    @Override
    public void error400(HttpClientErrorException error) {
        LOG.error(error.getMessage(), error);
    }

}
